<? // $Id: juke.php,v 1.2 2003/01/23 07:17:29 sunday Exp $

$juke = juke_connect();

function juke_connect() {
    global $banner;
    $s = socket_create( AF_INET, SOCK_STREAM, SOL_TCP );
    if( !$s ) {
        juke_throw_error( socket_strerror( socket_last_error() ) );
    }

    if( !socket_connect( $s, "localhost", 8008 ) ) {
        juke_throw_error( socket_strerror( socket_last_error() ) );
    }

    $banner = juke_read_line( $s );
    return $s;
}

function juke_add_playlist( $s, $album, $track = false ) {
    $q = sprintf( "ADD ALBUM %d", $album );
    if( $track ) {
        $q .= sprintf( " TRACK %d", $track );
    }
    juke_send_command( $s, $q );
}

function juke_play_index( $s, $index ) {
    juke_send_command( $s, sprintf( "PLAY INDEX %d", $index ) );
}

function juke_resume( $s ) {
    juke_send_command( $s, "PLAY" );
}

function juke_pause( $s ) {
    juke_send_command( $s, "PAUSE" );
}

function juke_status( $s ) {
    juke_write( $s, "STATUS\n" );
    $line = juke_read_line( $s );
    if( preg_match( '/^(\w{4}) (\d+) ([\d\.]+)/', $line, $m ) === 1 ) {
        $stat = array( 
          'state' => $m[1],
          'index' => $m[2],
          'time' => $m[3] );
        $line = juke_read_line( $s );
        return $stat;
    } 
    else {
        juke_throw_error( "Invalid STATUS response." );
    }
  
}

function juke_remove_playlist( $s, $index ) {
    juke_send_command( $s, sprintf( "REMOVE INDEX %d", $index ) );
}

function juke_move_playlist( $s, $index, $new ) {
    juke_send_command( $s, sprintf( "MOVE OLD %d NEW %d", $index, $new ));
}

function juke_clear_playlist( $s ) {
    juke_send_command( $s, "CLEAR" );
}

function juke_shuffle_playlist( $s ) {
    juke_send_command( $s, "SHUFFLE" );
}

function juke_get_playlist( $s ) {
    juke_write( $s, "LIST\n" );
    return juke_parse_track_list( $s );
}

function juke_album_tracklist( $s, $album_id ) {
    return juke_search_tracks( $s, false, false, false, $album_id );
}

function juke_search_tracks( $s, $artist, $title, $album_title, $album_id = false ) {
    $q = "";
 
    if( $artist ) {
        $q .= " ARTIST \"" . addslashes( $artist ) . "\"";
    }

    if( $title ) {
        $q .= " TITLE \"" . addslashes( $title ) . "\"";
    }

    if( $album_title ) {
        $q .= " ATITLE \"" . addslashes( $album_title ) . "\"";
    }

    if( $album_id ) {
        $q .= sprintf( " ALBUM %d", $album_id );
    }

    if( !$q ) {
        return false;
    }

    $q = "TRACK$q\n";

    juke_write( $s, $q );
    return juke_parse_track_list( $s );
}

function juke_parse_track_list( $s ) {
    $tracks = array();
    $line = juke_read_line( $s );
    while( $line && !preg_match( "/^\+OK\n/", $line ) ) {
        if( preg_match( 
          "/^(\d+) (\d+) \"((?:\\\"|[^\"])*)\" \"((?:\\\"|[^\"])*)\" \"((?:\\\"|[^\"])*)\" (\d+)/", 
          $line, 
          $m ) !== 1 ) {
            juke_throw_error( "Invalid track entry" );
        }
        $t = array( 
          'album' => $m[1], 
          'track' => $m[2], 
          'album_title' => stripslashes( $m[3] ),
          'title' => stripslashes( $m[4] ),
          'artist' => stripslashes( $m[5] ),
          'length' => $m[6] );
        $tracks[] = $t;
        $line = juke_read_line( $s );
    }
    return $tracks;
}

function juke_search_albums( $s, $artist, $title, $album_id = false ) {
    $q = "";
    if( $artist ) {
        $q .= " ARTIST \"" . addslashes( $artist ) . "\"";
    }

    if( $title ) {
        $q .= " TITLE \"" . addslashes( $title ) . "\"";
    }

    if( $album_id ) {
        $q .= sprintf( " ALBUM %d", $album_id );
    }

    juke_write( $s, "ALBUM$q\n" );
    return juke_parse_album_list( $s );
}

function juke_parse_album_list( $s ) {
    $albums = array();
    $line = juke_read_line( $s );
    while( $line && !preg_match( "/^\+OK\n/", $line ) ) {
        if( preg_match( 
          "/^(\d+) \"((?:\\\"|[^\"])*)\" \"((?:\\\"|[^\"])*)\" (\d+) (\d+)/", 
          $line, 
          $m ) ) {
            $a = array( 
              'album' => $m[1], 
              'title' => stripslashes( $m[2] ),
              'artist' => stripslashes( $m[3] ),
              'length' => $m[4],
              'tracks' => $m[5] );
            $albums[] = $a;
        }
        else {
            juke_throw_error( "Invalid Album entry" );
        }
        $line = juke_read_line( $s );
    }
    return $albums;
}

function juke_read_line( $s ) {
    $line = "";
    while( 1 ) {
        $buf = socket_read( $s, 1 );
        if( $buf === false ) {
            juke_throw_error( socket_strerror( $socket_last_error() ) );
        }
        $line .= $buf;
        if( $buf === "\n" ) {
            break;
        }
     }
     return $line;
}

function juke_write( $s, $cmd ) {
    $x = 0;
    while( $x < strlen( $cmd ) ) {
        $r = socket_write( $s, $cmd );
        if( $r === false ) {
            juke_throw_error( socket_strerror( $socket_last_error() ) );
        }
        $x += $r;
    }
    return $x;
}

function juke_send_command( $s, $cmd ) {
    juke_write( $s, "$cmd\n" );
    $line = juke_read_line( $s, $cmd );
    if( preg_match( "/^\+OK/", $line ) !== 1 ) {
        juke_throw_error( "$cmd: $line" );
    }
}

function juke_throw_error( $error ) {
    // This could be nicified. I'm lazy.
    echo "Error: $error";
    exit;
}

function shorten( $string, $len = 15 ) {
    if( strlen( $string ) <= $len + 3 ) {
        return htmlspecialchars( $string );
    }
    $short = substr( $string, 0, $len ) . ". . .";
    return "<span title=\"" . htmlspecialchars( $string ) . "\">" . htmlspecialchars( $short ) . "</span>";
}

?>
