/* $Id: ClientSocket.h,v 1.1.1.1 2003/01/23 05:29:37 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Class to handle input from a connected socket, queue up
 *   incoming traffic until a line is found, and parse that line.
 *   Created by ListenSocket.
 *
 * Note:
 *   This should really be broken into ClientSocket and
 *   JukeClientSocket, with the generic client stuff
 *   abstracted out for future use.
 */

#ifndef __CLIENTSOCKET_H__
#define __CLIENTSOCKET_H__
#include <vector>
#include "ArgVector.h"
#include "Album.h"
#include "Song.h"

class ClientSocket {
  public:

  // Constructor
  // Arguments: File descriptor returned by accept()

  ClientSocket( int file_descriptor );

  // Destructor
  ~ClientSocket();

   // Return the raw file descriptor
   int getFd() const;

   // Read data from the socket and process if possible.
   // Unprocessed data is kept in buf to try again on the next
   // read.
   // Returns non-zero if the socket closed.
   int read();

   // Write any pending data out to the socket.
   // Returns non-zero if socket closed.
   int write();

   // True if class has data to send to the remote side.
   int hasOutput() const;

   void SetNotify( unsigned int n );

   private:

   // Remove any '\r' characters from the input stream if
   // the client is mistakenly sending them.
   void cleanBuf();

   // Parse the buffer into lines and process any pending commands.
   // Processed lines are removed from the buffer.
   // Returns non-zero if client closed.
   int processBuf();

   // Parse and process a command line. Returns non-zero if client closed.
   int processLine( char* line );

   // Buffer text to be returned to client
   void queueOutput( const char *output );

   void ListTracks( const vector<Song*> *list );
   void ListTracks( const ArgVector &v );
   void ListAlbums( const vector<Album*> *list );
   void ListAlbums( const ArgVector &v );
 
   static char * AddSlashes( const char *string );

   int fd;       // File descriptor
   char *buf;    // Input buffer
   int buf_len;  // Length of data in buf
   int buf_size; // Allocated length of buf
   unsigned int notify; // Notify flags

   char *obuf;   // Output buffer
   int obuf_len; // Length of output buffer

};

#endif
