/* $Id: Database.C,v 1.1.1.1 2003/01/23 05:29:37 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Class to handle database connection and queries.
 *
 */

#include "config.h"
#include "Database.h"

#define WHERE( a ) ((((*a)++)==0)?"WHERE":"AND")

Database::Database() :
    conn( DBSTRING )
{
    if( conn.ConnectionBad() ) {
        cerr << "Error on DB Connect\n";
        exit( 1 );
    }
}

Database::~Database() {
    
}

Song *Database::CreateSong( unsigned int album, unsigned int track )
{
    char query[256];

    sprintf( query, 
      "SELECT album_title, title, artist, date_part( 'epoch', length ) " \
      "FROM track_detail WHERE album = %d AND track = %d", 
      album, track );

    if( conn.ExecTuplesOk( query ) && conn.Tuples() ) {
        Song *p = new Song( album, track, 
          conn.GetValue( 0, 0 ),
          conn.GetValue( 0, 1 ),
          conn.GetValue( 0, 2 ),
          atoi( conn.GetValue( 0, 3 ) ) );
        return p;
    }
    return NULL;
}

vector<Song*> *Database::SearchTracks( const ArgVector &argv )
{
    char *query = NULL;
    const char *album = argv.HashFind( "ALBUM" );
    const char *track = argv.HashFind( "TRACK" );
    const char *artist = argv.HashFind( "ARTIST" );
    const char *title = argv.HashFind( "TITLE" );
    const char *atitle = argv.HashFind( "ATITLE" );

    vector<Song*> *v = NULL;

    int and = 0;
    char *where = NULL;
    int x;
    if( album ) {
        x = WhereInt( &where, "album", atoi( album ), &and );
    }

    if( track ) {
        x = WhereInt( &where, "track", atoi( track ), &and );
    }

    if( artist ) {
        x = WhereString( &where, "artist", artist, &and );
    }

    if( title ) {
        x = WhereString( &where, "title", title, &and );
    }

    if( atitle ) {
        x = WhereString( &where, "album_title", atitle, &and );
    }

    asprintf( &query, 
      "SELECT album, track, album_title, title, artist, " \
      "date_part( 'epoch', length ) FROM track_detail %s", 
      where?where:"" );
 
    if( conn.ExecTuplesOk( query ) && conn.Tuples() ) {
        v = new vector<Song*>;
        for( x = 0; x < conn.Tuples(); x++ ) {
            Song *p = new Song(
              atoi( conn.GetValue( x, 0 ) ),
              atoi( conn.GetValue( x, 1 ) ),
              conn.GetValue( x, 2 ),
              conn.GetValue( x, 3 ),
              conn.GetValue( x, 4 ),
              atoi( conn.GetValue( x, 5 ) ) );
            v->push_back( p );
        }
    }
    free( query );
    free( where );
    return v;
}

vector<Album*> *Database::SearchAlbums( const ArgVector &argv ) 
{
    char *query = NULL;
    const char *id = argv.HashFind( "ALBUM" );
    const char *artist = argv.HashFind( "ARTIST" );
    const char *title = argv.HashFind( "TITLE" );

    char *where = NULL;
    int and = 0;
    int x;
    vector<Album*> *v = NULL;

    if( id ) {
        x = WhereInt( &where, "album", atoi( id ), &and );
    }

    if( artist ) {
        x = WhereString( &where, "artist", artist, &and );
    }

    if( title ) {
        x = WhereString( &where, "title", title, &and );
    }

    x = asprintf( &query, "SELECT album, artist, title, " \
        "date_part( 'epoch', length), tracks FROM album_detail %s", 
        where?where:"" );
    if( conn.ExecTuplesOk( query ) && conn.Tuples() ) {
        v = new vector<Album*>;
        for( x = 0; x < conn.Tuples(); x++ ) {
            Album *a = new Album( 
              atoi( conn.GetValue( x, 0 ) ),
              conn.GetValue( x, 1 ),
              conn.GetValue( x, 2 ),
              atoi( conn.GetValue( x, 3 ) ),
              atoi( conn.GetValue( x, 4 ) ) );
            v->push_back( a );
        }
    }
    free( where );
    free( query );
    return v;
}

vector<Song*> *Database::LoadPlaylist( const char *name, unsigned int *index )
{
    char *query = NULL;
    int x = strlen( name ) * 2 + 1;
    char *name2 = (char*)malloc( x );
    x = PQescapeString( name2, name, x );
    vector<Song*> *v = NULL;

    asprintf( &query, "SELECT last FROM playlists WHERE name = '%s'", name2 );
    if( conn.ExecTuplesOk( query ) && conn.Tuples() ) {
        *index = atoi( conn.GetValue( 0, 0 ) );
    }
    else {
        free( query );
        free( name2 );
        cerr << "Could't load index." << endl;
        return NULL;
    } 
    
    free( query );
    asprintf( &query, 
      "SELECT a.album, a.track, a.album_title, a.title, a.artist, " \
      "date_part( 'epoch', a.length ) FROM track_detail a, playlist_entries p " \
      "WHERE p.name = '%s' AND a.album = p.album AND a.track = p.track " \
      "ORDER BY p.position", name2 );
 
    if( conn.ExecTuplesOk( query ) && conn.Tuples() ) {
        v = new vector<Song*>;
        for( x = 0; x < conn.Tuples(); x++ ) {
            Song *p = new Song(
              atoi( conn.GetValue( x, 0 ) ),
              atoi( conn.GetValue( x, 1 ) ),
              conn.GetValue( x, 2 ),
              conn.GetValue( x, 3 ),
              conn.GetValue( x, 4 ),
              atoi( conn.GetValue( x, 5 ) ) );
            v->push_back( p );
        }
    }
    else if( !conn.ExecTuplesOk( query ) ) {
        cerr << "Could't load entries." << endl;
    }
    free( name2 );
    free( query );
    return v;
}

int Database::SavePlaylist( const char *name, vector<Song*> *list, unsigned int index )
{
    char *query = NULL;
    int x = strlen( name ) * 2 + 1;
    unsigned int i;
    char *name2 = (char*)malloc( x );
    x = PQescapeString( name2, name, x );

    if( !conn.ExecCommandOk( "BEGIN" ) ) {
        cerr << "Couldn't start transaction.\n";
        return -1;
    }

    asprintf( &query, "DELETE FROM playlists WHERE name = '%s'", name2 );
    if( !conn.ExecCommandOk( query ) ) {
        free( query );
        cerr << "Delete playlist failed.\n";
        conn.ExecCommandOk( "ROLLBACK" );
        return -1;
    }
    free( query );

    asprintf( &query, "INSERT INTO playlists( name, last ) " \
      "VALUES( '%s', %d )", name2, index );
    if( !conn.ExecCommandOk( query ) ) {
        free( query );
        cerr << "Insert playlist failed.\n";
        conn.ExecCommandOk( "ROLLBACK" );
        return -1;
    }
    free( query );

    for( i = 0; i < list->size(); i++ ) {
        asprintf( &query, 
          "INSERT INTO playlist_entries( name, position, album, track ) " \
          "VALUES( '%s', %d, %d, %d)", name2, i, 
          (*list)[i]->GetAlbum(), (*list)[i]->GetTrack() );
        if( !conn.ExecCommandOk( query ) ) {
            free( query );
            cerr << "Insert failed.\n";
            conn.ExecCommandOk( "ROLLBACK" );
            return -1;
        }
        free( query );
    }
    conn.ExecCommandOk( "COMMIT" ); 
    return 0;
}

int Database::WhereString( char **where, const char *column,
  const char *value, int *prefix ) {
    int x;
    int l = (*where)?strlen( *where ):0;
    char *scratch, *scratch2;

    x = strlen( value ) * 2 + 1;
    scratch = (char*)malloc( x );
    x = PQescapeString( scratch, value, x );

    x = asprintf( &scratch2, "%s %s ILIKE '%s' ", WHERE(prefix), 
      column, scratch );
    l += x + 1;
    if( *where ) {
        *where = (char*)realloc( *where, l );
        strlcat( *where, scratch2, l );
        free( scratch2 );
    } else {
        *where = scratch2;
    }
    free( scratch );
    return l;
}

int Database::WhereInt( char **where, const char *column,
  int value, int *prefix ) {
    int x;
    int l = (*where)?strlen( *where ):0;
    char *scratch = NULL;

    x = asprintf( &scratch, "%s %s=%d ", WHERE(prefix), column, value );
    l += x + 1;
    if( *where ) {
        *where = (char*)realloc( *where, l );
        strlcat( *where, scratch, l );
        free( scratch );
    } else {
        *where = scratch;
    }
    return l;
}

