/* $Id: ListenSocket6.C,v 1.1.1.1 2003/01/23 05:29:37 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   ListenSocket for IPv6/4 traffic
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>

#include "ListenSocket6.h"

ListenSocket6::ListenSocket6( unsigned int port ) throw( ListenSocket6::Error ):
    ListenSocket::ListenSocket( PF_INET6, SOCK_STREAM )
{
    struct sockaddr_in6 saddr;
    bzero( &saddr, sizeof( saddr ) );
    saddr.sin6_len = sizeof( saddr );
    saddr.sin6_family = AF_INET6;
    saddr.sin6_port = htons( port );
    // saddr.sin6_addr = IN6ADDR_ANY_INIT;
    
    if( bind( getFd(), (struct sockaddr*)&saddr, sizeof( saddr ) ) ) {
        perror( "bind()" );
        throw( Error( errno ) );
    }

    if( listen( getFd(), 5 ) ) {
        perror( "listen()" );
        throw( Error( errno ) );
    }
}

ClientSocket* ListenSocket6::accept() {
    struct sockaddr_in6 saddr;
    socklen_t alen( sizeof( saddr ) );
    int s = ::accept( getFd(), (struct sockaddr*)&saddr, &alen );
    #ifdef _DEBUG
    printf( "Accepted Connection\n" );
    #endif
    return new ClientSocket( s );
}

