/* $Id: Playlist.h,v 1.1.1.1 2003/01/23 05:29:37 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Class to manage playlist and feed current track to decoder.
 *
 */

#ifndef __PLAYLIST_H__
#define __PLAYLIST_H__

#include <pthread.h>
#include <vector>

#include "Decode.h"
#include "Song.h"

class Playlist {
  public:
    Playlist( const char *persist = "last" );
    ~Playlist();

  public:
    void LoadPlaylist( const char *name );
    void SavePlaylist( const char *name );
    void Shuffle();
    void QueueTrack( Song *track );
    void QueueTrack( const vector<Song*> *tracks );
    void Remove( unsigned int index );

    void Clear();

    void Move( unsigned int index, unsigned int newindex );

    void Next();
    void ChangeTrack( unsigned int index );

    int Status( int *index, double *position );

    void Pause();
    void Resume();

    unsigned int GetNotify();

    const vector<Song*>* GetList() const;

  private:
    // Associated Decoder instance
    OggDecoder d;

    // The name of the playlist to save back to on destruction
    char *persist;

    // Current track index
    unsigned int current_track;

    // Notify flags
    #define NOTIFY_PLAYLIST 1
    unsigned int notify;

    // Pthread mutex for list operations
    pthread_mutex_t list_mutex;

    // Playlist itself
    vector<Song*> list;
};

#endif
