#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "config.h"
#include "Playlist.h"
#include "ClientSocket.h"
#include "ListenSocket6.h"
#include "Database.h"

#include "sound.h"

Database db;
Playlist playlist;

void run_input();

/* 
 * Catch certain signals and call exit(0). The default handlers
 * don't attempt to do any sort of cleanup. Calling exit() makes
 * sure the destructor for playlist gets called and the playlist
 * gets dumped back to the database.
 */
void kill_handle( int sig ) {
    exit( 0 );
}

int main() {
    signal( SIGINT, kill_handle );
    signal( SIGHUP, kill_handle );

    if( sound_init() != 0 ) {
        fprintf( stderr, "Couldn't start sound.\n" );
        exit( 1 );
    }

    run_input();

    return 0;
}

void run_input() {

    fd_set r, w;
    unsigned int notify;
    vector<ClientSocket *>clients;

    struct timeval tv;
    try {
        ListenSocket6 ls( JUKEPORT );
        while( 1 ) {
            tv.tv_sec = 5;
            tv.tv_usec = 0;
         
            vector<ClientSocket *>::iterator it;
            int maxfd = ls.getFd();

            FD_ZERO( &r );
            FD_ZERO( &w );

            FD_SET( ls.getFd(), &r );

            if( !clients.empty() ) {
                for( it = clients.begin(); it != clients.end(); ++it ) {
                    FD_SET( (*it)->getFd(), &r );
                    FD_SET( (*it)->getFd(), &w );
                    maxfd = ((*it)->getFd() > maxfd)?(*it)->getFd():maxfd;
                 }
            }
            
            select( maxfd + 1, &r, &w, NULL, NULL );

            if( FD_ISSET( ls.getFd(), &r ) ) {
                ClientSocket *s = ls.accept();
                clients.push_back( s );
            }

            if( !clients.empty() ) {
                for( it = clients.begin(); (!clients.empty()) && 
                     (it != clients.end()); ) {
                    vector< ClientSocket* >::iterator next = it + 1;
                    if( FD_ISSET( (*it)->getFd(), &r ) ) {
                        if( (*it)->read() ) {
                            delete *it;
                            clients.erase( it );
                            continue;
                        }
                    }
                    it = next;
                }
            }
            if( !clients.empty() ) {
                for( it = clients.begin(); (!clients.empty()) && 
                     (it != clients.end()); ) {
                    vector< ClientSocket* >::iterator next = it + 1;
                    if( FD_ISSET( (*it)->getFd(), &w )  && (*it)->hasOutput() )
{
                        if( (*it)->write() ) {
                            delete *it;
                            clients.erase( it );
                            continue;
                        }
                    }
                    it = next;
                }
            }

            // Set notify flags
            if( (notify = playlist.GetNotify()) ) {
                for( it = clients.begin(); (!clients.empty()) &&
                     (it != clients.end()); it = it + 1 ) {
                    (*it)->SetNotify( notify );
                }
            } 
        }
    }
    catch( ListenSocket::Error e ) {
        printf( "Bailing\n" );
        exit( 1 );
    }
}
