/* $Id: sound.c,v 1.1.1.1 2003/01/23 05:29:37 sunday Exp $
 * Copyright (c) 2002, 2003 Joe Sunday <sunday@csh.rit.edu>
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Description:
 *   Stub file for FreeBSD /dev/dsp output
 *
 */

#include <sys/soundcard.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

#include "sound.h"

static int sounddev = 0;

int sound_init() {
    int x, y;

    sounddev = open( "/dev/dsp", O_RDWR );

    if( sounddev < 0 ) {
        perror( "open" );
        return -1;
    }
  
    /* Setup 16 bit, signed */
    x = AFMT_S16_LE;
    y = ioctl( sounddev, SNDCTL_DSP_SETFMT, &x);

    return 0;
}

int sound_setup( int channels, int rate ) {
    if( sounddev < 0 ) {
        return -1;
    }

    ioctl( sounddev, SNDCTL_DSP_CHANNELS, &channels );
    ioctl( sounddev, SNDCTL_DSP_SPEED, &rate ); 
    return 0;
}

int sound_play( char *buf, int len ) {
    return write( sounddev, buf, len );
}
    


